# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-08 18:37
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Layer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True, verbose_name='Name')),
                ('provider', jsonfield.fields.JSONField(default=dict, verbose_name='Provider')),
                ('projection', models.CharField(blank=True, max_length=128, null=True, verbose_name='Projection')),
                ('metatile', jsonfield.fields.JSONField(null=True, verbose_name='Metatile')),
                ('state_lock_timeout', models.PositiveIntegerField(null=True, verbose_name='State Lock Timeout')),
                ('cache_lifespan', models.PositiveIntegerField(null=True, verbose_name='State Lock Timeout')),
                ('write_cache', models.NullBooleanField(verbose_name='Write Cache?')),
                ('bounds', jsonfield.fields.JSONField(null=True, verbose_name='Bounds')),
                ('allowed_origin', models.CharField(max_length=256, null=True, verbose_name='Allowed Origin')),
                ('preview', jsonfield.fields.JSONField(null=True, verbose_name='Preview')),
                ('max_cache_age', models.PositiveIntegerField(null=True, verbose_name='Max Cache Age (seconds)')),
                ('redirects', jsonfield.fields.JSONField(null=True, verbose_name='Redirects')),
                ('tile_height', models.PositiveIntegerField(null=True, verbose_name='Tile Height')),
            ],
            options={
                'verbose_name': 'Layer',
                'verbose_name_plural': 'Layers',
            },
        ),
    ]
